@@ -11,7 +11,7 @@ import Data.String
 
 import Data.ByteString as B
 import Data.ByteString.Char8 as C8
-import Data.ByteString.Internal (c2w )
+import Data.ByteString.Internal (c2w)
 
 import Data.ByteString.Unsafe(unsafeIndex)
 import Data.Monoid
@@ -27,34 +27,34 @@ import Raaz.Core.Encode.Internal
 -- `IsString` instance. The combinators `fromBase16` and `showBase16`
 -- are exposed mainly to make these definitions easy.
 --
--- The base16 encoding only produces valid hex characters. However, to
--- aid easy presentation of long hexadecimal strings, a user can add
--- add arbitrary amount of spaces, newlines and the character ':'. The
--- decoding ignores these characters.
+
 newtype Base16 = Base16 {unBase16 :: ByteString} deriving (Eq, Monoid)
 
 -- Developers note: Internally base16 just stores the bytestring as
 -- is. The conversion happens when we do an encode and decode of
 -- actual base16.
+--
+
 
 instance Encodable Base16 where
   toByteString          = hex . unBase16
 
   fromByteString bs
-    | odd (B.length bs) = Nothing
-    | badCharacter bs   = Nothing
-    | otherwise         = Just $ Base16 $ unsafeFromHex bs
-    where badCharacter  = C8.any (not . isHexDigit)
-
-  unsafeFromByteString  = Base16 . unsafeFromHex
-
+    | B.length bs `mod` 2 /= 0 = Nothing
+    | validInput bs            = Just $ Base16 $ unsafeFromHex bs
+    | otherwise                = Nothing
+    where validInput  = C8.all isHexDigit
+  unsafeFromByteString bs
+    | B.length bs `mod` 2 /= 0 = error "base16 encoding is always of even size"
+    | otherwise                = Base16 $ unsafeFromHex bs
 
 instance Show Base16 where
   show = C8.unpack . toByteString
 
+-- | Ignores spaces and ':' (colon).
 instance IsString Base16 where
-  fromString = unsafeFromByteString . fromString
-
+  fromString = unsafeFromByteString . C8.filter (not . useless) . fromString
+    where useless c = isSpace c || c == ':'
 
 instance Format Base16 where
   encodeByteString = Base16
@@ -63,8 +63,13 @@ instance Format Base16 where
   decodeFormat     = unBase16
   {-# INLINE decodeFormat #-}
 
--- TODO: Since the encoding to base16 is usually used for user interaction
+-- Since the encoding to base16 is usually used for user interaction
 -- we can afford to be slower here.
+--
+-- TODO (Liquid Haskell)
+--
+{--@ hex :: inp:ByteString -> { bs : ByteString | bslen bs = 2 * bslen inp } @-}
+--
 hex :: ByteString -> ByteString
 hex  bs = fst $ B.unfoldrN (2 * B.length bs) gen 0
     where gen i | rm == 0   = Just (hexDigit $ top4 w, i+1)
@@ -72,6 +77,10 @@ hex  bs = fst $ B.unfoldrN (2 * B.length bs) gen 0
             where (idx, rm) = quotRem i 2
                   w         = unsafeIndex bs idx
 
+
+
+
+
 hexDigit :: Word8 -> Word8
 hexDigit x | x < 10    = c2w '0' + x
            | otherwise = c2w 'a' + (x - 10)
@@ -79,15 +88,9 @@ hexDigit x | x < 10    = c2w '0' + x
 top4 :: Word8 -> Word8; top4 x  = x `shiftR` 4
 bot4 :: Word8 -> Word8; bot4 x  = x  .&. 0x0F
 
-
+{-@ unsafeFromHex :: {bs : ByteString | (bslen bs) mod 2 == 0 } -> ByteString @-}
 unsafeFromHex :: ByteString -> ByteString
-unsafeFromHex  = unsafeFromHexP . C8.filter (not . useless)
-  where useless c = isSpace c || c == ':'
-
-unsafeFromHexP :: ByteString -> ByteString
-unsafeFromHexP bs
-  | odd (B.length bs) = error "base16 encoding is always of even size"
-  | otherwise         = fst $ B.unfoldrN len gen 0
+unsafeFromHex bs = fst $ B.unfoldrN len gen 0
   where len   = B.length bs `quot` 2
         gen i = Just (shiftL w0 4 .|. w1, i + 1)
           where w0 = fromHexWord $ unsafeIndex bs (2 * i)
