@@ -17,10 +17,7 @@ import Raaz.Core.Encode.Internal
 
 
 -- | The type corresponding to the standard padded base-64 binary
--- encoding.  The base-64 encoding only produces valid base-64
--- characters. However, to aid easy presentation of long base-64
--- strings, a user can add add arbitrary amount of spaces and
--- newlines. The decoding ignores these characters.
+-- encoding.
 newtype Base64 = Base64 {unBase64 :: ByteString} deriving (Eq, Monoid)
 
 -- Developers note: Internally base16 just stores the bytestring as
@@ -33,16 +30,12 @@ instance Encodable Base64 where
   fromByteString bs
     | B.null bs                = Just $ Base64 B.empty
     | B.length bs `rem` 4 /= 0 = Nothing
-    | badCharacter bs'         = Nothing
-    | not (isB64OrPad pl)      = Nothing
-    | not (isB64OrPad pf)      = Nothing
-    | otherwise                = Just $ Base64 $ unsafeFromB64 bs
-    where pl           = C8.last bs
-          pf           = C8.last $ C8.init bs
-          bs'          = C8.init $ C8.init bs
-          badCharacter = C8.any (not . isB64Char)
-          isB64Char c  = isAlpha c || isDigit c || c == '+' || c == '/'
-          isB64OrPad c = isB64Char c || c == '='
+    | okeyPad                  = Just $ Base64 $ unsafeFromB64 bs
+    | otherwise                = Nothing
+    where padPart     = C8.dropWhile isB64Char bs
+          okeyPad     = padPart == C8.empty || padPart == C8.singleton '=' || padPart == C8.pack "=="
+          isB64Char c = isAlpha c || isDigit c || c == '+' || c == '/'
+
 
   unsafeFromByteString bs | B.null bs = Base64 B.empty
                           | otherwise = Base64 $ unsafeFromB64 bs
@@ -51,9 +44,9 @@ instance Encodable Base64 where
 instance Show Base64 where
   show = C8.unpack . toByteString
 
+-- | Ignores spaces and newlines.
 instance IsString Base64 where
-  fromString = unsafeFromByteString . fromString
-
+  fromString = unsafeFromByteString . C8.filter (not . isSpace) . fromString
 
 instance Format Base64 where
   encodeByteString = Base64
@@ -111,8 +104,15 @@ unB64 w | c2w 'A' <= w && w <= c2w 'Z' = w - c2w 'A'
         | otherwise                    = error $ "oops unB64:" ++ [w2c w]
 
 
--- TODO: Since the encoding to base16 is usually used for user interaction
+
+
+-- Since the encoding to base16 is usually used for user interaction
 -- we can afford to be slower here.
+
+-- TODO (Liquid Haskell)
+--
+{--@ toB64 :: ByteString -> { bs : ByteString | (bslen bs) mod 4 == 0 @-}
+--
 toB64 :: ByteString -> ByteString
 toB64 bs = fst (B.unfoldrN (4*n) gen 0) <> padding
     where gen i    = Just (byte i, i + 1)
@@ -152,11 +152,7 @@ merg2 a b = (unB64 a `shiftL` 6) .|. unB64 b
 
 
 unsafeFromB64 :: ByteString -> ByteString
-unsafeFromB64 = unsafeFromB64P . C8.filter (not . useless)
-  where useless = isSpace
-
-unsafeFromB64P :: ByteString -> ByteString
-unsafeFromB64P bs = fst (B.unfoldrN (3*n) gen 0) <> unPad
+unsafeFromB64 bs = fst (B.unfoldrN (3*n) gen 0) <> unPad
   where n         = B.length bs `quot` 4 - 1
         gen i     = Just (byte i, i + 1)
         at blk i  = unsafeIndex bs $ 4 * blk + i
