@@ -2,7 +2,7 @@
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE ForeignFunctionInterface   #-}
-{-# LANGUAGE CPP                        #-}
+
 
 -- | This module exposes types that builds in type safety into some of
 -- the low level pointer operations. The functions here are pretty low
@@ -106,13 +106,13 @@ class (Enum u, Monoid u) => LengthUnit u where
 -- | Type safe lengths/offsets in units of bytes.
 newtype BYTES a  = BYTES a
         deriving ( Show, Eq, Equality, Ord, Enum, Integral
-                 , Real, Num, Storable
+                 , Real, Num, Storable, Bounded
                  )
 
 -- | Type safe lengths/offsets in units of bits.
 newtype BITS  a  = BITS  a
         deriving ( Show, Eq, Equality, Ord, Enum, Integral
-                 , Real, Num, Storable
+                 , Real, Num, Storable, Bounded
                  )
 
 -- | Type safe length unit that measures offsets in multiples of word
@@ -169,7 +169,7 @@ atLeastAligned l a = n + pad - 1
   where n = atLeast l
         -- Alignment adjusted to word boundary.
         algn = wordAlignment   <> a
-        pad  = atLeast $ BYTES  $ unAlignment $ algn
+        pad  = atLeast $ BYTES  $ unAlignment algn
 
 
 -- | Express length unit @src@ in terms of length unit @dest@ rounding
@@ -285,6 +285,18 @@ peekAligned = peek . nextAlignedPtr
 pokeAligned     :: Storable a => Ptr a -> a -> IO ()
 pokeAligned ptr =  poke $ nextAlignedPtr ptr
 
+-------------------------- Allocation  ---------------------------
+
+-- | A less general version of `allocaAligned` where the pointer passed
+-- is aligned to word boundary.
+allocaBuffer :: LengthUnit l
+             => l                  -- ^ buffer length
+             -> (Pointer -> IO b)  -- ^ the action to run
+             -> IO b
+{-# INLINE allocaBuffer #-}
+allocaBuffer = allocaAligned wordAlignment
+
+
 -- | The expression @allocaAligned a l action@ allocates a local
 -- buffer of length @l@ and alignment @a@ and passes it on to the IO
 -- action @action@. No explicit freeing of the memory is required as
@@ -300,6 +312,22 @@ allocaAligned algn l = allocaBytesAligned b a
   where BYTES     b = inBytes l
         Alignment a = algn
 
+----------------- Secure allocation ---------------------------------
+
+-- | A less general version of `allocaSecureAligned` where the pointer passed
+-- is aligned to word boundary
+allocaSecure :: LengthUnit l
+             => l
+             -> (Pointer -> IO b)
+             -> IO b
+allocaSecure = allocaSecureAligned wordAlignment
+
+foreign import ccall unsafe "raaz/core/memory.h raazMemorylock"
+  c_mlock :: Pointer -> BYTES Int -> IO Int
+
+foreign import ccall unsafe "raaz/core/memory.h raazMemoryunlock"
+  c_munlock :: Pointer -> BYTES Int -> IO ()
+
 
 -- | This function allocates a chunk of "secure" memory of a given
 -- size and runs the action. The memory (1) exists for the duration of
@@ -322,14 +350,8 @@ allocaSecureAligned :: LengthUnit l
                     -> (Pointer -> IO a)
                     -> IO a
 
-#ifdef HAVE_MLOCK
-
-foreign import ccall unsafe "sys/mman.h mlock"
-  c_mlock :: Pointer -> BYTES Int -> IO Int
 
 
-foreign import ccall unsafe "sys/mman.h munlock"
-  c_munlock :: Pointer -> BYTES Int -> IO ()
 
 allocaSecureAligned a l action = allocaAligned a l actualAction
   where sz = inBytes l
@@ -339,27 +361,6 @@ allocaSecureAligned a l action = allocaAligned a l actualAction
           releaseIt =  memset cptr 0 l >>  c_munlock cptr sz
           in bracket_ lockIt releaseIt $ action cptr
 
-#else
-allocaSecureAligned _ _ = fail "memory locking not supported on this platform"
-
-#endif
--- | A less general version of `allocaAligned` where the pointer passed
--- is aligned to word boundary.
-allocaBuffer :: LengthUnit l
-             => l                  -- ^ buffer length
-             -> (Pointer -> IO b)  -- ^ the action to run
-             -> IO b
-{-# INLINE allocaBuffer #-}
-allocaBuffer = allocaAligned wordAlignment
-
--- | A less general version of `allocaSecureAligned` where the pointer passed
--- is aligned to word boundary
-allocaSecure :: LengthUnit l
-             => l
-             -> (Pointer -> IO b)
-             -> IO b
-allocaSecure = allocaSecureAligned wordAlignment
-
 -- | Creates a memory of given size. It is better to use over
 -- @`mallocBytes`@ as it uses typesafe length.
 mallocBuffer :: LengthUnit l
