@@ -88,8 +88,8 @@ instance (Hash h, Recommendation h) => EndianStore (HMACKey h) where
   load             = peek
   adjustEndian _ _ = return ()
 
-instance (Hash h, Recommendation h) => Random (HMACKey h) where
-  random = unsafeStorableRandom
+instance (Hash h, Recommendation h) => RandomStorable (HMACKey h) where
+  fillRandomElements = unsafeFillRandomElements
 
 instance (Hash h, Recommendation h) => Encodable (HMACKey h)
 
@@ -130,18 +130,18 @@ hmac key = unsafePerformIO . hmacSource key
 {-# SPECIALIZE hmac :: (Hash h, Recommendation h) => Key (HMAC h) -> B.ByteString -> HMAC h #-}
 {-# SPECIALIZE hmac :: (Hash h, Recommendation h) => Key (HMAC h) -> L.ByteString -> HMAC h #-}
 
--- | Compute the hash of file.
+-- | Compute the hmac of file.
 hmacFile :: (Hash h, Recommendation h)
-         => Key (HMAC h)
-         -> FilePath  -- ^ File to be hashed
+         => Key (HMAC h) -- ^ Key to use for mac-ing
+         -> FilePath     -- ^ File to be hashed
          -> IO (HMAC h)
 hmacFile key fileName = withBinaryFile fileName ReadMode $ hmacSource key
 {-# INLINEABLE hmacFile #-}
 
--- | Compute the hash of a generic byte source.
+-- | Compute the hmac of a generic byte source.
 hmacSource :: ( Hash h, Recommendation h, ByteSource src )
-           => Key (HMAC h)
-           -> src  -- ^ Message
+           => Key (HMAC h)  -- ^ key to use for mac-ing.
+           -> src           -- ^ Message
            -> IO (HMAC h)
 hmacSource = go undefined
   where go :: (Hash h, Recommendation h, ByteSource src)
@@ -152,7 +152,7 @@ hmacSource = go undefined
 {-# SPECIALIZE hmacSource :: (Hash h, Recommendation h) => Key (HMAC h) -> Handle -> IO (HMAC h) #-}
 
 
--- | Compute the hash of a pure byte source like, `B.ByteString`.
+-- | Compute the hmac of a pure byte source like, `B.ByteString`.
 hmac' :: ( Hash h, Recommendation h, PureByteSource src )
       => Implementation h
       -> Key (HMAC h)
@@ -174,7 +174,7 @@ hmac' impl key = unsafePerformIO . hmacSource' impl key
   #-}
 
 
--- | Compute the hash of file.
+-- | Compute the hmac of file.
 hmacFile' :: (Hash h, Recommendation h)
          => Implementation h
          -> Key (HMAC h)
@@ -183,7 +183,7 @@ hmacFile' :: (Hash h, Recommendation h)
 hmacFile' impl key fileName = withBinaryFile fileName ReadMode $ hmacSource' impl key
 {-# INLINEABLE hmacFile' #-}
 
-
+-- | Compute the hmac of a generic ByteSource using a given implementation.
 hmacSource' :: (Hash h, Recommendation h, ByteSource src)
             => Implementation h
             -> Key (HMAC h)
@@ -196,7 +196,7 @@ hmacSource' imp@(SomeHashI hI) key src =
     initialise ()
     allocate $ \ ptr -> do
       liftIO $ unsafeCopyToPointer innerFirstBlock ptr
-      compress hI ptr 1
+      compress hI ptr $ toEnum 1
 
     -- Finish it by hashing the source.
     innerHash <- completeHashing hI src
@@ -206,12 +206,12 @@ hmacSource' imp@(SomeHashI hI) key src =
     initialise ()
     allocate $ \ ptr -> do
       liftIO $ unsafeCopyToPointer outerFirstBlock ptr
-      compress hI ptr 1
+      compress hI ptr $ toEnum 1
 
     -- Finish it with hashing the  hash computed above
     HMAC <$> completeHashing hI (toByteString innerHash)
 
-  where allocate = liftPointerAction $ allocBufferFor imp $ 1 `asTypeOf` (theBlock key)
+  where allocate = liftPointerAction $ allocBufferFor imp $ (toEnum 1) `asTypeOf` (theBlock key)
         innerFirstBlock = B.map (xor 0x36) $ hmacAdjustKey key
         outerFirstBlock = B.map (xor 0x5c) $ hmacAdjustKey key
         theBlock :: Key (HMAC h) -> BLOCKS h
